#define Version "2020_0915"

// MCU : PIC 16F887
// External clock : Quartz 4 MHz
// Driving of 2 servomotors : - servomotor LOWER (360 turn) on portd.b0 (pin 19),
//                              > Servomotor TOWER PRO MG995 (360),
//                              > Timer 0,
//                            - servomotor UPPER (180/270 turn) on portd.b1 (pin 20),
//                              > Servomotor TOWER PRO MG996R (180/270),
//                              > Timer 1.
// LED is connected to portc.b2 (pin 17).

//  ----------------------------------------------------------------------------
unsigned int  i, j, k;              // unsigned int : 0 to 65535
unsigned int Cnt0,Cnt1;
unsigned int Consigne_1, Consigne_0;     // unsigned int : 0 to 65535
unsigned int joystick_X, joystick_Y;

volatile unsigned char Flag_TMR0;
volatile unsigned char Flag_TMR1;
volatile unsigned char Flag_TMR2;

char CRam1[80];

// ---------------------------------------------------------------------------

void ADC_initialization()
{
 ANSELH=0;
 ANSEL=0b00000011;                 // c'est plus sur !  voies analogique sur RA0 et RA1
 ADCON0 = 0b01000001;              // b0 = 1 (ADON) : le convertisseur A/N interne est activ.
                                   // b1 (GO/DONE)  : A/D Conversion Status bit.
                                   // b2, b3, b4, b5 (CHS0, CHS1, CHS2, CHS3) : Analog Channel Select bits.
                                   // b6 = 1 (ADSC0), b7 = 0 (ADSC1)          : fosc / 8 = 4 MHz / 8 = 0,5 MHz = 500 KHz
                                   //                                           tosc = 1 / fosc = 1 / 500 KHz = 0,002 mS = 2 uS.
                                   // 2 uS est le temps de conversion A/D d'un bit, TAD.
                                   // Pour une conversion totale sur 10 bits il faut 12 TAD.
                                   // Pour une conversion correcte il faut que TAD = 1,6 uS au minimum.
                                   // TAD = 2 uS / bit. 12 TAD = 24 uS pour 10 bits.

 ADCON1 = 0b10000000;              // b0 = b1 = b2 = b3 = 0 : Unimplemented. Read as '0'.
                                   // b4 = 0 (VCFG0)        : Voltage reference VDD.
                                   // b5 = 0 (VCFG1)        : Voltage reference VSS.
                                   // b6 = 0                : Unimplemented. Read as '0'.
                                   // b7 = 1                : ADFM (A/D result ForMat) = 1 (format justifi  droite).
                                   // Explication :
                                   //          ADRESH        |         ADRESL
                                   // b7|b6|b5|b4|b3|b2|b1|b0|b7|b6|b5|b4|b3|b2|b1|b0
                                   //  0| 0| 0| 0| 0| 0| r| r| r| r| r| r| r| r| r| r
                                   //    les 6 MSB = 0 |   rsultat r sur 10 bits
}

unsigned int ADC_read(unsigned char channel)
// Si message d'erreur "'ADC_read' Identifier redefined" pendant compilation, dans Library Manager dcocher ADC_Read.
// ADC_read est ma propre routine crite pour apprentissage. Je n'utilise pas ADC_Read de la librairie Mikroelektronika.
{
 static unsigned int k;                   // Variable locale.
 ADCON0 = 0x41 | (channel<<2);     // ADCON0 = 01000001
                                   //        & 11000011 (masque)
                                   // ADCON0 = 01000001
                                   // masque :
                                   // b0 = 1 (ADON)                               : pour prendre en compte l'tat du convertisseur A/D, en service (0) ou  l'arrt (1).
                                   // b1 = 1 (GO/DONE)                            : pour prendre en compte l'tat de la conversion A/D, en cours (1) ou termine (0).
                                   // b2 = b3 = b4 = b5 = 0 (CHS0 CHS1 CHS2 CHS3) : pour re initialiser au premier canal de conversion, le canal 0 (CHS0 = CHS1 = CHS2 = CHS3 = 0).
                                   // b6 = b7 = 1 (ADSC0 ADSC1)                   : pour prendre en compte la valeur du diviseur, de la vitesse de conversion, choisi.
                                   // Dcalage channel de 2 bits  gauche pour placer la valeur de channel dans bits b2 b3 b4 b5 (CHS0 CHS1 CHS2 CHS3).
 Delay_ms(2);                      // Dlais de 2 mS minimum. >>> A ajuster si besoin. <<<
 ADCON0.GO_DONE = 1;               // Dclenchement de la conversion A/N. via ADCON0.b2 = 1;
 _asm NOP;                         // Recommand par Microchip.
 while (ADCON0.GO_DONE == 1);      // Attendre que le bit GO.DONE passe  0.
 k = ADRESH <<8;   
 k=  k | ADRESL;
 return(k);
 }


/* ----------------------------------------------------------------------------*/

// Timer2 (8 bits) : 20 mS period
// Prescaler 1:16
// Postscaler 1:5
// PR2 preload = 250
// TMR2 preload = 0
// Interrupt time : 20,005 mS

void Init_Timer2(void)
{
 // Timer 2 in timer mode :
 T2CON = 0x26;
 PR2 = 250;             // Timer2 8 bits register is preloaded with 250 in order to count from 250 to 256, 5 cycles of 20 mS = 100 mS,
                        // divided by postscaler value of 5 = 20 mS.
 TMR2IE_bit = 1;        // Sets Interrupt Enable bit to 1.
 INTCON = 0x40;         // GIE = 0.
 Flag_TMR2 = 0;
}

// Timer1 (16 bits) : Ton servomotor UPPER
void Init_Timer1(void)
{
 // Timer 1 in timer mode :
 //Prescaler 1:1; TMR1 Preload = 64536; Actual Interrupt Time : 1 ms    at 4MHz
 T1CON = 0x01;          // T1CON register > b0 > TMR1ON bit = 1 > Timer1 is enabled.
                        //                > b1 > TMR1CS bit = 0 > Timer1 counts pulses from internal oscillator.
 TMR1IF_bit = 0;        // Sets Interrupt Flag bit to 0.
 TMR1H = 0xFC;          // Timer1 16 bits register is preloaded with 64536 in order to count from 64536 to 65536, 1000 cycles of 1 uS = 1 mS.
 TMR1L = 0x18;          //
 TMR1IE_bit = 0;        // Sets Interrupt Enable bit to 1.
 INTCON = 0xC0;         // INTCON register > b6 > PEIE bit = 1 > All unmasked peripheral interrupts are enabled.
                        //                 > b7 > GIE bit = 1 > All unmasked interrupts are enabled.
 Flag_TMR1 = 0;         // Flag init Timer1.
}


// Timer0 (8 bits) : Ton servomotor LOWER
//Timer0
//Prescaler 1:1; TMR0 Preload = 246; Actual Interrupt Time : 10 us
void Init_Timer0()
{
 // Timer 0 in timer mode :
 OPTION_REG = 0x08;     // OPTION_REG register > b0 > PS0 bit = 0 > Prescaler rate : 1:1.
                        //                     > b3 > PSA bit = 1 > Prescaler is assigned to the wachdog timer in order to have prescaler rate of 1:1.
                        //                     > b7 > RBPU bit = 1 > PORTB pull-up resistors are disabled.
 TMR0 = 246;            // Timer0 8 bits register is preloaded with 156 in order to count from 156 to 256, 100 cycles of 1 uS = 100 uS = 0,1 mS.
 INTCON = 0xA0;         // INTCON register > b2 > TOIF bit = 0 > Resets the bit in order to allow TMR0 to count pulses from internal oscillator.
                        //                 > b5 > TOIE bit = 0 > TMR0 overflow interrupt is disabled.
                        //                 > b7 > GIE bit = 0 > All unmasked interrupts are disabled.
 Cnt0 = 0;              // Resets "10 uS" counter.
 Flag_TMR0 = 0;
}

/* ----------------------------------------------------------------------------*/

void Interrupt()
{
unsigned int Scaled_TMR1_Target;
 if (TMR2IF_bit && TMR2IE_bit)      // After 20 mS elapsed with Timer2 ...
    {

     TMR0IE_bit=0;
     TMR1IE_bit=0;
     // Timer 0 for Servomotor LOWER.
      // Lance Timer0 cal sur 10 et devant compter Cn_t x 10S
     TMR0 = 246;        // Timer0 8 bits register is preloaded with 156 in order to count from 156 to 256, 100 cycles of 1 uS = 100 uS = 0,1 mS.
     Flag_TMR0 = 0;
     Cnt0=0;     //// consigne_0 =ADC0 vallue traite   => 100=> 100x10S =1mS   150=> 150x10S => 1,5mS  200=> 200*10S=>  2mS
     PORTD.B0 = 1;                  // Servomotor LOWER ON.
     PORTC.B0 = 1;                  // LED ON.

     // TMR1 for  Servomotor UPPER .
      Cnt1=0  ;   // consigne_1      65535-(ADC value traitee);   // consigne_1    1000 , 1500, 2000 (S)
      T1CON = 0x00;          // T1CON register > b0 > TMR1ON bit = 1 > Timer1 is disable.
      TMR1IF_bit = 0;        // Sets Interrupt Flag bit to 0.
      TMR1H = Cnt1 >>8;
      TMR1L= Cnt1 & 0x00FF;  // Timer1 16 bits register is preloaded .
      Flag_TMR1 = 0;                 // reset Flag Timer1
      PORTD.B1 = 1;                  // Servomotor UPPER ON.
      PORTC.B1 = 1;                  // LED ON.
     
      INTCON = 0xC0;         // INTCON register > b6 > PEIE bit = 1 > All unmasked peripheral interrupts are enabled.
      TMR1ON_bit=1 ;         // lance Timer1
      TMR0IE_bit=1;         // autorise IT Timer0
      TMR1IE_bit=1;          // autorise IT Timer0

     // Restarts Timer 2 :
     TMR2IF_bit = 0;          // Resets Interrupt Flag.
     PR2 = 250;               // can be omitted, because autoreload  PR2
     //Flag_TMR2 = 0;
    }

 if (TMR0IF_bit && TMR0IE_bit)      // After 100 uS elapsed with TMR0 ...
    {
     TMR0IF_bit = 0;
     TMR0 = 246;
     Cnt0++;                        // Increments "10 uS" counter.
     if (Cnt0 > Consigne_0)        //  Cnt0 > (10S * Consigne_0 )
     {
     Cnt0=0;
     Flag_TMR0=1;                    // Timer0 stops interrupt.
     TMR0IE_bit=0;
     PORTD.B0 = 1;                  // Servomotor LOWER OFF.
     PORTC.B0 = 1;                  // LED OFF.
     }
    }
 // Servomotor UPPER.
 if (TMR1IF_bit && TMR1IE_bit)      // After n x 1uS elapsed with TMR1 ...
    {
     TMR1ON_bit=0;
     TMR1IF_bit = 0;
     TMR1IE_bit=0;
     Scaled_TMR1_Target=   64535- Consigne_1;
     TMR1L = Scaled_TMR1_Target & 0x00FF;
     TMR1H = Scaled_TMR1_Target >> 8;
     Flag_TMR1 = 1;                 // Timer1 stops counting & interrupt.
     PORTD.B1 = 0;                  // Servomotor UPPER OFF.
     PORTC.B1 = 0;                  // LED OFF.

    }
}

void CRLF1(void)
{
    UART1_Write(13);
      UART1_Write(10);
}

void Print( char *T)
{
  while(*(T++)>0)
  {
   UART1_Write(*(T));
   }
 }

void CPrint(const char *T1)
{
  while(*(T1++)>0)
  {
   UART1_Write(*(T1));
   }
 }

/* ----------------------------------------------------------------------------*/

sbit Servo_LOWER at PORTD.B0  ;
sbit Servo_LOWER_dir at TRISD.B0 ;
sbit Servo_UPPER at PORTD.B1   ;
sbit Servo_UPPER_dir  at TRISD.B1 ;

sbit Led_LOWER at PORTC.B0  ;
sbit Led_LOWER_dir at TRISC.B0 ;
sbit Led_UPPER at PORTC.B1   ;
sbit Led_UPPER_dir  at TRISC.B1 ;

void main()
{
 ANSEL=0;
 TRISC = 0;                               // PORT C configur en sortie.
 PORTC = 0;                               // Initialisation du PORT C  0.
 TRISD = 0;                               // PORT D configur en sortie.
 PORTD = 0;                               // Initialisation du PORT D  0.
 Servo_LOWER_dir=0;
 Servo_UPPER_dir=0;
 Led_LOWER_dir=0;
 Led_LOWER_dir=0;

 // Servomotor LOWER OFF.
  Servo_LOWER=0;
  Led_LOWER=0;
  // Servomotor UPPER OFF.
  Servo_UPPER=0;
  Led_UPPER=0;

 ADC_initialization();
 UART1_Init(19200);
 CPrint(" Test 2 servo , MCU= 16F887 4MHz  \r\n");
 CPrint(" Servomotor LOWER 270 : TOWER PRO M996R (270) \r\n");
 CPrint(" Servomotor UPPER x360 : TOWER PRO M996R modifi \r\n");
 CPrint(" Compiler: MikroC 7.60   version application : "Version"\r\n");
 CRLF1();

  Init_Timer0();  // IT disable
  Init_Timer1(); // IT disable
  Init_Timer2(); // IT Arme
  GIE_bit=1;
  
 CPrint(" Valeur Consignes\r\n ");
  
 while(1)
 {
  joystick_X = (ADC_read(0));             // Reads Y axis value with channel 0 (AN0) on pin #2.    0 1023
  Consigne_0 = 100 + (joystick_X / 10);  //  100 +0=  100 *10S => 1mS      [100 + (1023/10)] * 10S => 2,02mS
   CPrint(" Lower Servo  = ");
   WordToStr(Consigne_0,CRam1);
   Print(CRam1);
   CPrint("   Upper Servo = ");
    joystick_Y = (ADC_read(1));             // Reads Y axis value with channel 1 (AN1) on pin #3.
   Consigne_1 = 64535-  joystick_Y ;       //   1000  2000 S
    WordToStr(Consigne_1,CRam1);
   Print(CRam1);
   CRLF1();
   Delay_ms(1000);   // delay  reduire si besoin
 }
}